Index: uspace/drv/bus/usb/usbhid/kbd/kbddev.h
===================================================================
--- uspace/drv/bus/usb/usbhid/kbd/kbddev.h	(revision c69646f882a7a12ae81b85a589c3f95239f23e8d)
+++ uspace/drv/bus/usb/usbhid/kbd/kbddev.h	(revision 5da719915787705f661397f14e439ebeb2aaf2f1)
@@ -38,7 +38,6 @@
 
 #include <stdint.h>
-
+#include <async.h>
 #include <fibril_synch.h>
-
 #include <usb/hid/hid.h>
 #include <usb/hid/hidparser.h>
@@ -58,5 +57,5 @@
  * data, such as currently pressed keys, modifiers and lock keys.
  *
- * Also holds a IPC phone to the console (since there is now no other way to 
+ * Also holds a IPC session to the console (since there is now no other way to 
  * communicate with it).
  *
@@ -83,6 +82,6 @@
 	unsigned lock_keys;
 	
-	/** IPC phone to the console device (for sending key events). */
-	int console_phone;
+	/** IPC session to the console device (for sending key events). */
+	async_sess_t *console_sess;
 	
 	/** @todo What is this actually? */
Index: uspace/drv/bus/usb/usbhid/mouse/mousedev.h
===================================================================
--- uspace/drv/bus/usb/usbhid/mouse/mousedev.h	(revision c69646f882a7a12ae81b85a589c3f95239f23e8d)
+++ uspace/drv/bus/usb/usbhid/mouse/mousedev.h	(revision 5da719915787705f661397f14e439ebeb2aaf2f1)
@@ -38,4 +38,5 @@
 
 #include <usb/dev/driver.h>
+#include <async.h>
 
 struct usb_hid_dev;
@@ -45,7 +46,7 @@
 /** Container for USB mouse device. */
 typedef struct {
-	/** IPC phone to console (consumer). */
-	int mouse_phone;
-	int wheel_phone;
+	/** IPC session to console (consumer). */
+	async_sess_t *mouse_sess;
+	async_sess_t *wheel_sess;
 	
 	int32_t *buttons;
Index: uspace/lib/c/Makefile
===================================================================
--- uspace/lib/c/Makefile	(revision c69646f882a7a12ae81b85a589c3f95239f23e8d)
+++ uspace/lib/c/Makefile	(revision 5da719915787705f661397f14e439ebeb2aaf2f1)
@@ -97,7 +97,5 @@
 	generic/ipc.c \
 	generic/ns.c \
-	generic/ns_obsolete.c \
 	generic/async.c \
-	generic/async_obsolete.c \
 	generic/loader.c \
 	generic/getopt.c \
Index: uspace/lib/c/generic/async.c
===================================================================
--- uspace/lib/c/generic/async.c	(revision c69646f882a7a12ae81b85a589c3f95239f23e8d)
+++ uspace/lib/c/generic/async.c	(revision 5da719915787705f661397f14e439ebeb2aaf2f1)
@@ -118,4 +118,52 @@
 #define CONN_HASH_TABLE_BUCKETS    32
 
+/** Session data */
+struct async_sess {
+	/** List of inactive exchanges */
+	list_t exch_list;
+	
+	/** Exchange management style */
+	exch_mgmt_t mgmt;
+	
+	/** Session identification */
+	int phone;
+	
+	/** First clone connection argument */
+	sysarg_t arg1;
+	
+	/** Second clone connection argument */
+	sysarg_t arg2;
+	
+	/** Third clone connection argument */
+	sysarg_t arg3;
+	
+	/** Exchange mutex */
+	fibril_mutex_t mutex;
+	
+	/** Number of opened exchanges */
+	atomic_t refcnt;
+	
+	/** Mutex for stateful connections */
+	fibril_mutex_t remote_state_mtx;
+	
+	/** Data for stateful connections */
+	void *remote_state_data;
+};
+
+/** Exchange data */
+struct async_exch {
+	/** Link into list of inactive exchanges */
+	link_t sess_link;
+	
+	/** Link into global list of inactive exchanges */
+	link_t global_link;
+	
+	/** Session pointer */
+	async_sess_t *sess;
+	
+	/** Exchange identification */
+	int phone;
+};
+
 /** Async framework global futex */
 atomic_t async_futex = FUTEX_INITIALIZER;
@@ -134,4 +182,17 @@
 	ipc_call_t call;
 } msg_t;
+
+/** Message data */
+typedef struct {
+	awaiter_t wdata;
+	
+	/** If reply was received. */
+	bool done;
+	
+	/** Pointer to where the answer data is stored. */
+	ipc_call_t *dataptr;
+	
+	sysarg_t retval;
+} amsg_t;
 
 /* Client connection data */
Index: uspace/lib/c/generic/async_obsolete.c
===================================================================
--- uspace/lib/c/generic/async_obsolete.c	(revision c69646f882a7a12ae81b85a589c3f95239f23e8d)
+++ 	(revision )
@@ -1,450 +1,0 @@
-/*
- * Copyright (c) 2006 Ondrej Palkovsky
- * All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * - Redistributions of source code must retain the above copyright
- *   notice, this list of conditions and the following disclaimer.
- * - Redistributions in binary form must reproduce the above copyright
- *   notice, this list of conditions and the following disclaimer in the
- *   documentation and/or other materials provided with the distribution.
- * - The name of the author may not be used to endorse or promote products
- *   derived from this software without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
- * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
- * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
- * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
- * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
- * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- */
-
-/** @addtogroup libc
- * @{
- */
-/** @file
- */
-
-#define LIBC_ASYNC_C_
-#define LIBC_ASYNC_OBSOLETE_C_
-#include <ipc/ipc.h>
-#include <async.h>
-#include <async_obsolete.h>
-#include "private/async.h"
-#undef LIBC_ASYNC_C_
-#undef LIBC_ASYNC_OBSOLETE_C_
-
-#include <fibril.h>
-#include <malloc.h>
-#include <errno.h>
-
-/** Send message and return id of the sent message.
- *
- * The return value can be used as input for async_wait() to wait for
- * completion.
- *
- * @param phoneid Handle of the phone that will be used for the send.
- * @param method  Service-defined method.
- * @param arg1    Service-defined payload argument.
- * @param arg2    Service-defined payload argument.
- * @param arg3    Service-defined payload argument.
- * @param arg4    Service-defined payload argument.
- * @param dataptr If non-NULL, storage where the reply data will be
- *                stored.
- *
- * @return Hash of the sent message or 0 on error.
- *
- */
-aid_t async_obsolete_send_fast(int phoneid, sysarg_t method, sysarg_t arg1,
-    sysarg_t arg2, sysarg_t arg3, sysarg_t arg4, ipc_call_t *dataptr)
-{
-	amsg_t *msg = malloc(sizeof(amsg_t));
-	
-	if (!msg)
-		return 0;
-	
-	msg->done = false;
-	msg->dataptr = dataptr;
-	
-	msg->wdata.to_event.inlist = false;
-	
-	/*
-	 * We may sleep in the next method,
-	 * but it will use its own means
-	 */
-	msg->wdata.active = true;
-	
-	ipc_call_async_4(phoneid, method, arg1, arg2, arg3, arg4, msg,
-	    reply_received, true);
-	
-	return (aid_t) msg;
-}
-
-/** Pseudo-synchronous message sending - fast version.
- *
- * Send message asynchronously and return only after the reply arrives.
- *
- * This function can only transfer 4 register payload arguments. For
- * transferring more arguments, see the slower async_req_slow().
- *
- * @param phoneid Hash of the phone through which to make the call.
- * @param method  Method of the call.
- * @param arg1    Service-defined payload argument.
- * @param arg2    Service-defined payload argument.
- * @param arg3    Service-defined payload argument.
- * @param arg4    Service-defined payload argument.
- * @param r1      If non-NULL, storage for the 1st reply argument.
- * @param r2      If non-NULL, storage for the 2nd reply argument.
- * @param r3      If non-NULL, storage for the 3rd reply argument.
- * @param r4      If non-NULL, storage for the 4th reply argument.
- * @param r5      If non-NULL, storage for the 5th reply argument.
- *
- * @return Return code of the reply or a negative error code.
- *
- */
-sysarg_t async_obsolete_req_fast(int phoneid, sysarg_t method, sysarg_t arg1,
-    sysarg_t arg2, sysarg_t arg3, sysarg_t arg4, sysarg_t *r1, sysarg_t *r2,
-    sysarg_t *r3, sysarg_t *r4, sysarg_t *r5)
-{
-	ipc_call_t result;
-	aid_t eid = async_obsolete_send_4(phoneid, method, arg1, arg2, arg3, arg4,
-	    &result);
-	
-	sysarg_t rc;
-	async_wait_for(eid, &rc);
-	
-	if (r1)
-		*r1 = IPC_GET_ARG1(result);
-	
-	if (r2)
-		*r2 = IPC_GET_ARG2(result);
-	
-	if (r3)
-		*r3 = IPC_GET_ARG3(result);
-	
-	if (r4)
-		*r4 = IPC_GET_ARG4(result);
-	
-	if (r5)
-		*r5 = IPC_GET_ARG5(result);
-	
-	return rc;
-}
-
-/** Wrapper for IPC_M_SHARE_OUT calls using the async framework.
- *
- * @param phoneid Phone that will be used to contact the receiving side.
- * @param src     Source address space area base address.
- * @param flags   Flags to be used for sharing. Bits can be only cleared.
- *
- * @return Zero on success or a negative error code from errno.h.
- *
- */
-int async_obsolete_share_out_start(int phoneid, void *src, unsigned int flags)
-{
-	return async_obsolete_req_3_0(phoneid, IPC_M_SHARE_OUT, (sysarg_t) src, 0,
-	    (sysarg_t) flags);
-}
-
-/** Wrapper for ipc_hangup.
- *
- * @param phone Phone handle to hung up.
- *
- * @return Zero on success or a negative error code.
- *
- */
-int async_obsolete_hangup(int phone)
-{
-	return ipc_hangup(phone);
-}
-
-/** Wrapper for IPC_M_DATA_WRITE calls using the async framework.
- *
- * @param phoneid Phone that will be used to contact the receiving side.
- * @param src     Address of the beginning of the source buffer.
- * @param size    Size of the source buffer.
- * @param flags   Flags to control the data transfer.
- *
- * @return Zero on success or a negative error code from errno.h.
- *
- */
-int async_obsolete_data_write_start_generic(int phoneid, const void *src, size_t size,
-    int flags)
-{
-	return async_obsolete_req_3_0(phoneid, IPC_M_DATA_WRITE, (sysarg_t) src,
-	    (sysarg_t) size, (sysarg_t) flags);
-}
-
-/** Start IPC_M_DATA_READ using the async framework.
- *
- * @param phoneid Phone that will be used to contact the receiving side.
- * @param dst     Address of the beginning of the destination buffer.
- * @param size    Size of the destination buffer (in bytes).
- * @param dataptr Storage of call data (arg 2 holds actual data size).
- *
- * @return Hash of the sent message or 0 on error.
- *
- */
-aid_t async_obsolete_data_read(int phone, void *dst, size_t size,
-    ipc_call_t *dataptr)
-{
-	return async_obsolete_send_2(phone, IPC_M_DATA_READ, (sysarg_t) dst,
-	    (sysarg_t) size, dataptr);
-}
-
-/** Wrapper for IPC_M_DATA_READ calls using the async framework.
- *
- * @param phoneid Phone that will be used to contact the receiving side.
- * @param dst     Address of the beginning of the destination buffer.
- * @param size    Size of the destination buffer.
- * @param flags   Flags to control the data transfer.
- *
- * @return Zero on success or a negative error code from errno.h.
- *
- */
-int async_obsolete_data_read_start_generic(int phoneid, void *dst, size_t size, int flags)
-{
-	return async_obsolete_req_3_0(phoneid, IPC_M_DATA_READ, (sysarg_t) dst,
-	    (sysarg_t) size, (sysarg_t) flags);
-}
-
-/** Wrapper for making IPC_M_CONNECT_TO_ME calls using the async framework.
- *
- * Ask through phone for a new connection to some service.
- *
- * @param phone           Phone handle used for contacting the other side.
- * @param arg1            User defined argument.
- * @param arg2            User defined argument.
- * @param arg3            User defined argument.
- * @param client_receiver Connection handing routine.
- *
- * @return New phone handle on success or a negative error code.
- *
- */
-int async_obsolete_connect_to_me(int phone, sysarg_t arg1, sysarg_t arg2,
-    sysarg_t arg3, async_client_conn_t client_receiver, void *carg)
-{
-	sysarg_t task_hash;
-	sysarg_t phone_hash;
-	int rc = async_obsolete_req_3_5(phone, IPC_M_CONNECT_TO_ME, arg1, arg2, arg3,
-	    NULL, NULL, NULL, &task_hash, &phone_hash);
-	if (rc != EOK)
-		return rc;
-	
-	if (client_receiver != NULL)
-		async_new_connection(task_hash, phone_hash, 0, NULL,
-		    client_receiver, carg);
-	
-	return EOK;
-}
-
-/** Wrapper for making IPC_M_CONNECT_ME_TO calls using the async framework.
- *
- * Ask through phone for a new connection to some service.
- *
- * @param phone Phone handle used for contacting the other side.
- * @param arg1  User defined argument.
- * @param arg2  User defined argument.
- * @param arg3  User defined argument.
- *
- * @return New phone handle on success or a negative error code.
- *
- */
-int async_obsolete_connect_me_to(int phone, sysarg_t arg1, sysarg_t arg2,
-    sysarg_t arg3)
-{
-	sysarg_t newphid;
-	int rc = async_obsolete_req_3_5(phone, IPC_M_CONNECT_ME_TO, arg1, arg2, arg3,
-	    NULL, NULL, NULL, NULL, &newphid);
-	
-	if (rc != EOK)
-		return rc;
-	
-	return newphid;
-}
-
-/** Wrapper for making IPC_M_CONNECT_ME_TO calls using the async framework.
- *
- * Ask through phone for a new connection to some service and block until
- * success.
- *
- * @param phoneid Phone handle used for contacting the other side.
- * @param arg1    User defined argument.
- * @param arg2    User defined argument.
- * @param arg3    User defined argument.
- *
- * @return New phone handle on success or a negative error code.
- *
- */
-int async_obsolete_connect_me_to_blocking(int phoneid, sysarg_t arg1, sysarg_t arg2,
-    sysarg_t arg3)
-{
-	sysarg_t newphid;
-	int rc = async_obsolete_req_4_5(phoneid, IPC_M_CONNECT_ME_TO, arg1, arg2, arg3,
-	    IPC_FLAG_BLOCKING, NULL, NULL, NULL, NULL, &newphid);
-	
-	if (rc != EOK)
-		return rc;
-	
-	return newphid;
-}
-
-/** Send message and return id of the sent message
- *
- * The return value can be used as input for async_wait() to wait for
- * completion.
- *
- * @param phoneid Handle of the phone that will be used for the send.
- * @param method  Service-defined method.
- * @param arg1    Service-defined payload argument.
- * @param arg2    Service-defined payload argument.
- * @param arg3    Service-defined payload argument.
- * @param arg4    Service-defined payload argument.
- * @param arg5    Service-defined payload argument.
- * @param dataptr If non-NULL, storage where the reply data will be
- *                stored.
- *
- * @return Hash of the sent message or 0 on error.
- *
- */
-aid_t async_obsolete_send_slow(int phoneid, sysarg_t method, sysarg_t arg1,
-    sysarg_t arg2, sysarg_t arg3, sysarg_t arg4, sysarg_t arg5,
-    ipc_call_t *dataptr)
-{
-	amsg_t *msg = malloc(sizeof(amsg_t));
-	
-	if (!msg)
-		return 0;
-	
-	msg->done = false;
-	msg->dataptr = dataptr;
-	
-	msg->wdata.to_event.inlist = false;
-	
-	/*
-	 * We may sleep in the next method,
-	 * but it will use its own means
-	 */
-	msg->wdata.active = true;
-	
-	ipc_call_async_5(phoneid, method, arg1, arg2, arg3, arg4, arg5, msg,
-	    reply_received, true);
-	
-	return (aid_t) msg;
-}
-
-void async_obsolete_msg_0(int phone, sysarg_t imethod)
-{
-	ipc_call_async_0(phone, imethod, NULL, NULL, true);
-}
-
-void async_obsolete_msg_1(int phone, sysarg_t imethod, sysarg_t arg1)
-{
-	ipc_call_async_1(phone, imethod, arg1, NULL, NULL, true);
-}
-
-void async_obsolete_msg_2(int phone, sysarg_t imethod, sysarg_t arg1, sysarg_t arg2)
-{
-	ipc_call_async_2(phone, imethod, arg1, arg2, NULL, NULL, true);
-}
-
-void async_obsolete_msg_3(int phone, sysarg_t imethod, sysarg_t arg1, sysarg_t arg2,
-    sysarg_t arg3)
-{
-	ipc_call_async_3(phone, imethod, arg1, arg2, arg3, NULL, NULL, true);
-}
-
-void async_obsolete_msg_4(int phone, sysarg_t imethod, sysarg_t arg1, sysarg_t arg2,
-    sysarg_t arg3, sysarg_t arg4)
-{
-	ipc_call_async_4(phone, imethod, arg1, arg2, arg3, arg4, NULL, NULL,
-	    true);
-}
-
-void async_obsolete_msg_5(int phone, sysarg_t imethod, sysarg_t arg1, sysarg_t arg2,
-    sysarg_t arg3, sysarg_t arg4, sysarg_t arg5)
-{
-	ipc_call_async_5(phone, imethod, arg1, arg2, arg3, arg4, arg5, NULL,
-	    NULL, true);
-}
-
-/** Wrapper for IPC_M_SHARE_IN calls using the async framework.
- *
- * @param phoneid Phone that will be used to contact the receiving side.
- * @param dst     Destination address space area base.
- * @param size    Size of the destination address space area.
- * @param arg     User defined argument.
- * @param flags   Storage for the received flags. Can be NULL.
- *
- * @return Zero on success or a negative error code from errno.h.
- *
- */
-int async_obsolete_share_in_start(int phoneid, void *dst, size_t size, sysarg_t arg,
-    unsigned int *flags)
-{
-	sysarg_t tmp_flags;
-	int res = async_obsolete_req_3_2(phoneid, IPC_M_SHARE_IN, (sysarg_t) dst,
-	    (sysarg_t) size, arg, NULL, &tmp_flags);
-	
-	if (flags)
-		*flags = (unsigned int) tmp_flags;
-	
-	return res;
-}
-
-int async_obsolete_forward_fast(ipc_callid_t callid, int phoneid, sysarg_t imethod,
-    sysarg_t arg1, sysarg_t arg2, unsigned int mode)
-{
-	return ipc_forward_fast(callid, phoneid, imethod, arg1, arg2, mode);
-}
-
-int async_obsolete_forward_slow(ipc_callid_t callid, int phoneid, sysarg_t imethod,
-    sysarg_t arg1, sysarg_t arg2, sysarg_t arg3, sysarg_t arg4, sysarg_t arg5,
-    unsigned int mode)
-{
-	return ipc_forward_slow(callid, phoneid, imethod, arg1, arg2, arg3, arg4,
-	    arg5, mode);
-}
-
-/** Wrapper for forwarding any data that is about to be received
- *
- */
-int async_obsolete_data_write_forward_fast(int phoneid, sysarg_t method, sysarg_t arg1,
-    sysarg_t arg2, sysarg_t arg3, sysarg_t arg4, ipc_call_t *dataptr)
-{
-	ipc_callid_t callid;
-	if (!async_data_write_receive(&callid, NULL)) {
-		ipc_answer_0(callid, EINVAL);
-		return EINVAL;
-	}
-	
-	aid_t msg = async_obsolete_send_fast(phoneid, method, arg1, arg2, arg3, arg4,
-	    dataptr);
-	if (msg == 0) {
-		ipc_answer_0(callid, EINVAL);
-		return EINVAL;
-	}
-	
-	int retval = ipc_forward_fast(callid, phoneid, 0, 0, 0,
-	    IPC_FF_ROUTE_FROM_ME);
-	if (retval != EOK) {
-		async_wait_for(msg, NULL);
-		ipc_answer_0(callid, retval);
-		return retval;
-	}
-	
-	sysarg_t rc;
-	async_wait_for(msg, &rc);
-	
-	return (int) rc;
-}
-
-/** @}
- */
Index: uspace/lib/c/generic/ns_obsolete.c
===================================================================
--- uspace/lib/c/generic/ns_obsolete.c	(revision c69646f882a7a12ae81b85a589c3f95239f23e8d)
+++ 	(revision )
@@ -1,51 +1,0 @@
-/*
- * Copyright (c) 2011 Martin Decky
- * All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * - Redistributions of source code must retain the above copyright
- *   notice, this list of conditions and the following disclaimer.
- * - Redistributions in binary form must reproduce the above copyright
- *   notice, this list of conditions and the following disclaimer in the
- *   documentation and/or other materials provided with the distribution.
- * - The name of the author may not be used to endorse or promote products
- *   derived from this software without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
- * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
- * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
- * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
- * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
- * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- */
-
-/** @addtogroup libc
- * @{
- */
-/** @file
- */
-
-#include <async.h>
-#include <async_obsolete.h>
-#include <ns_obsolete.h>
-#include <abi/ipc/methods.h>
-
-int service_obsolete_connect(sysarg_t service, sysarg_t arg2, sysarg_t arg3)
-{
-	return async_obsolete_connect_me_to(PHONE_NS, service, arg2, arg3);
-}
-
-int service_obsolete_connect_blocking(sysarg_t service, sysarg_t arg2, sysarg_t arg3)
-{
-	return async_obsolete_connect_me_to_blocking(PHONE_NS, service, arg2, arg3);
-}
-
-/** @}
- */
Index: uspace/lib/c/generic/private/async.h
===================================================================
--- uspace/lib/c/generic/private/async.h	(revision c69646f882a7a12ae81b85a589c3f95239f23e8d)
+++ uspace/lib/c/generic/private/async.h	(revision 5da719915787705f661397f14e439ebeb2aaf2f1)
@@ -43,52 +43,4 @@
 #include <bool.h>
 
-/** Session data */
-struct _async_sess {
-	/** List of inactive exchanges */
-	list_t exch_list;
-	
-	/** Exchange management style */
-	exch_mgmt_t mgmt;
-	
-	/** Session identification */
-	int phone;
-	
-	/** First clone connection argument */
-	sysarg_t arg1;
-	
-	/** Second clone connection argument */
-	sysarg_t arg2;
-	
-	/** Third clone connection argument */
-	sysarg_t arg3;
-	
-	/** Exchange mutex */
-	fibril_mutex_t mutex;
-	
-	/** Number of opened exchanges */
-	atomic_t refcnt;
-	
-	/** Mutex for stateful connections */
-	fibril_mutex_t remote_state_mtx;
-	
-	/** Data for stateful connections */
-	void *remote_state_data;
-};
-
-/** Exchange data */
-struct _async_exch {
-	/** Link into list of inactive exchanges */
-	link_t sess_link;
-	
-	/** Link into global list of inactive exchanges */
-	link_t global_link;
-	
-	/** Session pointer */
-	async_sess_t *sess;
-	
-	/** Exchange identification */
-	int phone;
-};
-
 /** Structures of this type are used to track the timeout events. */
 typedef struct {
@@ -129,17 +81,4 @@
 } awaiter_t;
 
-/** Message data */
-typedef struct {
-	awaiter_t wdata;
-	
-	/** If reply was received. */
-	bool done;
-	
-	/** Pointer to where the answer data is stored. */
-	ipc_call_t *dataptr;
-	
-	sysarg_t retval;
-} amsg_t;
-
 extern void __async_init(void);
 extern void async_insert_timeout(awaiter_t *);
Index: uspace/lib/c/include/async.h
===================================================================
--- uspace/lib/c/include/async.h	(revision c69646f882a7a12ae81b85a589c3f95239f23e8d)
+++ uspace/lib/c/include/async.h	(revision 5da719915787705f661397f14e439ebeb2aaf2f1)
@@ -96,9 +96,9 @@
 
 /** Forward declarations */
-struct _async_exch;
-struct _async_sess;
-
-typedef struct _async_sess async_sess_t;
-typedef struct _async_exch async_exch_t;
+struct async_exch;
+struct async_sess;
+
+typedef struct async_sess async_sess_t;
+typedef struct async_exch async_exch_t;
 
 extern atomic_t threads_in_ipc_wait;
Index: uspace/lib/c/include/async_obsolete.h
===================================================================
--- uspace/lib/c/include/async_obsolete.h	(revision c69646f882a7a12ae81b85a589c3f95239f23e8d)
+++ 	(revision )
@@ -1,264 +1,0 @@
-/*
- * Copyright (c) 2006 Ondrej Palkovsky
- * All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * - Redistributions of source code must retain the above copyright
- *   notice, this list of conditions and the following disclaimer.
- * - Redistributions in binary form must reproduce the above copyright
- *   notice, this list of conditions and the following disclaimer in the
- *   documentation and/or other materials provided with the distribution.
- * - The name of the author may not be used to endorse or promote products
- *   derived from this software without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
- * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
- * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
- * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
- * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
- * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- */
-
-/** @addtogroup libc
- * @{
- */
-/** @file
- */
-
-#if ((defined(LIBC_IPC_H_)) && (!defined(LIBC_ASYNC_OBSOLETE_C_)))
-	#error Do not intermix low-level IPC interface and async framework
-#endif
-
-#ifndef LIBC_ASYNC_OBSOLETE_H_
-#define LIBC_ASYNC_OBSOLETE_H_
-
-#define async_obsolete_send_0(phoneid, method, dataptr) \
-	async_obsolete_send_fast((phoneid), (method), 0, 0, 0, 0, (dataptr))
-#define async_obsolete_send_1(phoneid, method, arg1, dataptr) \
-	async_obsolete_send_fast((phoneid), (method), (arg1), 0, 0, 0, (dataptr))
-#define async_obsolete_send_2(phoneid, method, arg1, arg2, dataptr) \
-	async_obsolete_send_fast((phoneid), (method), (arg1), (arg2), 0, 0, (dataptr))
-#define async_obsolete_send_3(phoneid, method, arg1, arg2, arg3, dataptr) \
-	async_obsolete_send_fast((phoneid), (method), (arg1), (arg2), (arg3), 0, (dataptr))
-#define async_obsolete_send_4(phoneid, method, arg1, arg2, arg3, arg4, dataptr) \
-	async_obsolete_send_fast((phoneid), (method), (arg1), (arg2), (arg3), (arg4), \
-	    (dataptr))
-#define async_obsolete_send_5(phoneid, method, arg1, arg2, arg3, arg4, arg5, dataptr) \
-	async_obsolete_send_slow((phoneid), (method), (arg1), (arg2), (arg3), (arg4), \
-	    (arg5), (dataptr))
-
-extern aid_t async_obsolete_send_fast(int phoneid, sysarg_t method, sysarg_t arg1,
-    sysarg_t arg2, sysarg_t arg3, sysarg_t arg4, ipc_call_t *dataptr);
-extern aid_t async_obsolete_send_slow(int phoneid, sysarg_t method, sysarg_t arg1,
-    sysarg_t arg2, sysarg_t arg3, sysarg_t arg4, sysarg_t arg5,
-    ipc_call_t *dataptr);
-
-#define async_obsolete_req_0_0(phoneid, method) \
-	async_obsolete_req_fast((phoneid), (method), 0, 0, 0, 0, NULL, NULL, NULL, NULL, \
-	    NULL)
-#define async_obsolete_req_0_1(phoneid, method, r1) \
-	async_obsolete_req_fast((phoneid), (method), 0, 0, 0, 0, (r1), NULL, NULL, NULL, \
-	    NULL)
-#define async_obsolete_req_0_2(phoneid, method, r1, r2) \
-	async_obsolete_req_fast((phoneid), (method), 0, 0, 0, 0, (r1), (r2), NULL, NULL, \
-	    NULL)
-#define async_obsolete_req_0_3(phoneid, method, r1, r2, r3) \
-	async_obsolete_req_fast((phoneid), (method), 0, 0, 0, 0, (r1), (r2), (r3), NULL, \
-	    NULL)
-#define async_obsolete_req_0_4(phoneid, method, r1, r2, r3, r4) \
-	async_obsolete_req_fast((phoneid), (method), 0, 0, 0, 0, (r1), (r2), (r3), (r4), \
-	    NULL)
-#define async_obsolete_req_0_5(phoneid, method, r1, r2, r3, r4, r5) \
-	async_obsolete_req_fast((phoneid), (method), 0, 0, 0, 0, (r1), (r2), (r3), (r4), \
-	    (r5))
-#define async_obsolete_req_1_0(phoneid, method, arg1) \
-	async_obsolete_req_fast((phoneid), (method), (arg1), 0, 0, 0, NULL, NULL, NULL, \
-	    NULL, NULL)
-#define async_obsolete_req_1_1(phoneid, method, arg1, rc1) \
-	async_obsolete_req_fast((phoneid), (method), (arg1), 0, 0, 0, (rc1), NULL, NULL, \
-	    NULL, NULL)
-#define async_obsolete_req_1_2(phoneid, method, arg1, rc1, rc2) \
-	async_obsolete_req_fast((phoneid), (method), (arg1), 0, 0, 0, (rc1), (rc2), NULL, \
-	    NULL, NULL)
-#define async_obsolete_req_1_3(phoneid, method, arg1, rc1, rc2, rc3) \
-	async_obsolete_req_fast((phoneid), (method), (arg1), 0, 0, 0, (rc1), (rc2), (rc3), \
-	    NULL, NULL)
-#define async_obsolete_req_1_4(phoneid, method, arg1, rc1, rc2, rc3, rc4) \
-	async_obsolete_req_fast((phoneid), (method), (arg1), 0, 0, 0, (rc1), (rc2), (rc3), \
-	    (rc4), NULL)
-#define async_obsolete_req_1_5(phoneid, method, arg1, rc1, rc2, rc3, rc4, rc5) \
-	async_obsolete_req_fast((phoneid), (method), (arg1), 0, 0, 0, (rc1), (rc2), (rc3), \
-	    (rc4), (rc5))
-#define async_obsolete_req_2_0(phoneid, method, arg1, arg2) \
-	async_obsolete_req_fast((phoneid), (method), (arg1), (arg2), 0, 0, NULL, NULL, \
-	    NULL, NULL, NULL)
-#define async_obsolete_req_2_1(phoneid, method, arg1, arg2, rc1) \
-	async_obsolete_req_fast((phoneid), (method), (arg1), (arg2), 0, 0, (rc1), NULL, \
-	    NULL, NULL, NULL)
-#define async_obsolete_req_2_2(phoneid, method, arg1, arg2, rc1, rc2) \
-	async_obsolete_req_fast((phoneid), (method), (arg1), (arg2), 0, 0, (rc1), (rc2), \
-	    NULL, NULL, NULL)
-#define async_obsolete_req_2_3(phoneid, method, arg1, arg2, rc1, rc2, rc3) \
-	async_obsolete_req_fast((phoneid), (method), (arg1), (arg2), 0, 0, (rc1), (rc2), \
-	    (rc3), NULL, NULL)
-#define async_obsolete_req_2_4(phoneid, method, arg1, arg2, rc1, rc2, rc3, rc4) \
-	async_obsolete_req_fast((phoneid), (method), (arg1), (arg2), 0, 0, (rc1), (rc2), \
-	    (rc3), (rc4), NULL)
-#define async_obsolete_req_2_5(phoneid, method, arg1, arg2, rc1, rc2, rc3, rc4, rc5) \
-	async_obsolete_req_fast((phoneid), (method), (arg1), (arg2), 0, 0, (rc1), (rc2), \
-	    (rc3), (rc4), (rc5))
-#define async_obsolete_req_3_0(phoneid, method, arg1, arg2, arg3) \
-	async_obsolete_req_fast((phoneid), (method), (arg1), (arg2), (arg3), 0, NULL, NULL, \
-	    NULL, NULL, NULL)
-#define async_obsolete_req_3_1(phoneid, method, arg1, arg2, arg3, rc1) \
-	async_obsolete_req_fast((phoneid), (method), (arg1), (arg2), (arg3), 0, (rc1), \
-	    NULL, NULL, NULL, NULL)
-#define async_obsolete_req_3_2(phoneid, method, arg1, arg2, arg3, rc1, rc2) \
-	async_obsolete_req_fast((phoneid), (method), (arg1), (arg2), (arg3), 0, (rc1), \
-	    (rc2), NULL, NULL, NULL)
-#define async_obsolete_req_3_3(phoneid, method, arg1, arg2, arg3, rc1, rc2, rc3) \
-	async_obsolete_req_fast((phoneid), (method), (arg1), (arg2), (arg3), 0, (rc1), \
-	    (rc2), (rc3), NULL, NULL)
-#define async_obsolete_req_3_4(phoneid, method, arg1, arg2, arg3, rc1, rc2, rc3, rc4) \
-	async_obsolete_req_fast((phoneid), (method), (arg1), (arg2), (arg3), 0, (rc1), \
-	    (rc2), (rc3), (rc4), NULL)
-#define async_obsolete_req_3_5(phoneid, method, arg1, arg2, arg3, rc1, rc2, rc3, rc4, \
-    rc5) \
-	async_obsolete_req_fast((phoneid), (method), (arg1), (arg2), (arg3), 0, (rc1), \
-	    (rc2), (rc3), (rc4), (rc5))
-#define async_obsolete_req_4_0(phoneid, method, arg1, arg2, arg3, arg4) \
-	async_obsolete_req_fast((phoneid), (method), (arg1), (arg2), (arg3), (arg4), NULL, \
-	    NULL, NULL, NULL, NULL)
-#define async_obsolete_req_4_1(phoneid, method, arg1, arg2, arg3, arg4, rc1) \
-	async_obsolete_req_fast((phoneid), (method), (arg1), (arg2), (arg3), (arg4), (rc1), \
-	    NULL, NULL, NULL, NULL)
-#define async_obsolete_req_4_2(phoneid, method, arg1, arg2, arg3, arg4, rc1, rc2) \
-	async_obsolete_req_fast((phoneid), (method), (arg1), (arg2), (arg3), (arg4), (rc1), \
-	    (rc2), NULL, NULL, NULL)
-#define async_obsolete_req_4_3(phoneid, method, arg1, arg2, arg3, arg4, rc1, rc2, rc3) \
-	async_obsolete_req_fast((phoneid), (method), (arg1), (arg2), (arg3), (arg4), (rc1), \
-	    (rc2), (rc3), NULL, NULL)
-#define async_obsolete_req_4_4(phoneid, method, arg1, arg2, arg3, arg4, rc1, rc2, rc3, \
-    rc4) \
-	async_obsolete_req_fast((phoneid), (method), (arg1), (arg2), (arg3), (arg4), \
-	    (rc1), (rc2), (rc3), (rc4), NULL)
-#define async_obsolete_req_4_5(phoneid, method, arg1, arg2, arg3, arg4, rc1, rc2, rc3, \
-    rc4, rc5) \
-	async_obsolete_req_fast((phoneid), (method), (arg1), (arg2), (arg3), (arg4), \
-	    (rc1), (rc2), (rc3), (rc4), (rc5))
-#define async_obsolete_req_5_0(phoneid, method, arg1, arg2, arg3, arg4, arg5) \
-	async_obsolete_req_slow((phoneid), (method), (arg1), (arg2), (arg3), (arg4), \
-	    (arg5), NULL, NULL, NULL, NULL, NULL)
-#define async_obsolete_req_5_1(phoneid, method, arg1, arg2, arg3, arg4, arg5, rc1) \
-	async_obsolete_req_slow((phoneid), (method), (arg1), (arg2), (arg3), (arg4), \
-	    (arg5), (rc1), NULL, NULL, NULL, NULL)
-#define async_obsolete_req_5_2(phoneid, method, arg1, arg2, arg3, arg4, arg5, rc1, rc2) \
-	async_obsolete_req_slow((phoneid), (method), (arg1), (arg2), (arg3), (arg4), \
-	    (arg5), (rc1), (rc2), NULL, NULL, NULL)
-#define async_obsolete_req_5_3(phoneid, method, arg1, arg2, arg3, arg4, arg5, rc1, rc2, \
-    rc3) \
-	async_obsolete_req_slow((phoneid), (method), (arg1), (arg2), (arg3), (arg4), \
-	    (arg5), (rc1), (rc2), (rc3), NULL, NULL)
-#define async_obsolete_req_5_4(phoneid, method, arg1, arg2, arg3, arg4, arg5, rc1, rc2, \
-    rc3, rc4) \
-	async_obsolete_req_slow((phoneid), (method), (arg1), (arg2), (arg3), (arg4), \
-	    (arg5), (rc1), (rc2), (rc3), (rc4), NULL)
-#define async_obsolete_req_5_5(phoneid, method, arg1, arg2, arg3, arg4, arg5, rc1, rc2, \
-    rc3, rc4, rc5) \
-	async_obsolete_req_slow((phoneid), (method), (arg1), (arg2), (arg3), (arg4), \
-	    (arg5), (rc1), (rc2), (rc3), (rc4), (rc5))
-
-extern sysarg_t async_obsolete_req_fast(int phoneid, sysarg_t method, sysarg_t arg1,
-    sysarg_t arg2, sysarg_t arg3, sysarg_t arg4, sysarg_t *r1, sysarg_t *r2,
-    sysarg_t *r3, sysarg_t *r4, sysarg_t *r5);
-extern sysarg_t async_obsolete_req_slow(int phoneid, sysarg_t method, sysarg_t arg1,
-    sysarg_t arg2, sysarg_t arg3, sysarg_t arg4, sysarg_t arg5, sysarg_t *r1,
-    sysarg_t *r2, sysarg_t *r3, sysarg_t *r4, sysarg_t *r5);
-
-extern void async_obsolete_msg_0(int, sysarg_t);
-extern void async_obsolete_msg_1(int, sysarg_t, sysarg_t);
-extern void async_obsolete_msg_2(int, sysarg_t, sysarg_t, sysarg_t);
-extern void async_obsolete_msg_3(int, sysarg_t, sysarg_t, sysarg_t, sysarg_t);
-extern void async_obsolete_msg_4(int, sysarg_t, sysarg_t, sysarg_t, sysarg_t, sysarg_t);
-extern void async_obsolete_msg_5(int, sysarg_t, sysarg_t, sysarg_t, sysarg_t, sysarg_t,
-    sysarg_t);
-
-extern int async_obsolete_forward_slow(ipc_callid_t, int, sysarg_t, sysarg_t, sysarg_t,
-    sysarg_t, sysarg_t, sysarg_t, unsigned int);
-
-extern int async_obsolete_connect_to_me(int, sysarg_t, sysarg_t, sysarg_t,
-    async_client_conn_t, void *);
-extern int async_obsolete_connect_me_to(int, sysarg_t, sysarg_t, sysarg_t);
-extern int async_obsolete_connect_me_to_blocking(int, sysarg_t, sysarg_t, sysarg_t);
-extern int async_obsolete_hangup(int);
-
-#define async_obsolete_share_in_start_0_0(phoneid, dst, size) \
-	async_obsolete_share_in_start((phoneid), (dst), (size), 0, NULL)
-#define async_obsolete_share_in_start_0_1(phoneid, dst, size, flags) \
-	async_obsolete_share_in_start((phoneid), (dst), (size), 0, (flags))
-#define async_obsolete_share_in_start_1_0(phoneid, dst, size, arg) \
-	async_obsolete_share_in_start((phoneid), (dst), (size), (arg), NULL)
-#define async_obsolete_share_in_start_1_1(phoneid, dst, size, arg, flags) \
-	async_obsolete_share_in_start((phoneid), (dst), (size), (arg), (flags))
-
-extern int async_obsolete_share_in_start(int, void *, size_t, sysarg_t,
-    unsigned int *);
-extern int async_obsolete_share_out_start(int, void *, unsigned int);
-
-#define async_obsolete_data_write_start(p, buf, len) \
-	async_obsolete_data_write_start_generic((p), (buf), (len), IPC_XF_NONE)
-
-#define async_obsolete_data_read_start(p, buf, len) \
-	async_obsolete_data_read_start_generic((p), (buf), (len), IPC_XF_NONE)
-
-#define async_obsolete_data_write_forward_0_0(phoneid, method, answer) \
-	async_obsolete_data_write_forward_fast((phoneid), (method), 0, 0, 0, 0, NULL)
-#define async_obsolete_data_write_forward_0_1(phoneid, method, answer) \
-	async_obsolete_data_write_forward_fast((phoneid), (method), 0, 0, 0, 0, (answer))
-#define async_obsolete_data_write_forward_1_0(phoneid, method, arg1, answer) \
-	async_obsolete_data_write_forward_fast((phoneid), (method), (arg1), 0, 0, 0, NULL)
-#define async_obsolete_data_write_forward_1_1(phoneid, method, arg1, answer) \
-	async_obsolete_data_write_forward_fast((phoneid), (method), (arg1), 0, 0, 0, \
-	    (answer))
-#define async_obsolete_data_write_forward_2_0(phoneid, method, arg1, arg2, answer) \
-	async_obsolete_data_write_forward_fast((phoneid), (method), (arg1), (arg2), 0, 0, \
-	    NULL)
-#define async_obsolete_data_write_forward_2_1(phoneid, method, arg1, arg2, answer) \
-	async_obsolete_data_write_forward_fast((phoneid), (method), (arg1), (arg2), 0, 0, \
-	    (answer))
-#define async_obsolete_data_write_forward_3_0(phoneid, method, arg1, arg2, arg3, answer) \
-	async_obsolete_data_write_forward_fast((phoneid), (method), (arg1), (arg2), (arg3), \
-	    0, NULL)
-#define async_obsolete_data_write_forward_3_1(phoneid, method, arg1, arg2, arg3, answer) \
-	async_obsolete_data_write_forward_fast((phoneid), (method), (arg1), (arg2), (arg3), \
-	    0, (answer))
-#define async_obsolete_data_write_forward_4_0(phoneid, method, arg1, arg2, arg3, arg4, answer) \
-	async_obsolete_data_write_forward_fast((phoneid), (method), (arg1), (arg2), (arg3), \
-	    (arg4), NULL)
-#define async_obsolete_data_write_forward_4_1(phoneid, method, arg1, arg2, arg3, arg4, answer) \
-	async_obsolete_data_write_forward_fast((phoneid), (method), (arg1), (arg2), (arg3), \
-	    (arg4), (answer))
-
-extern aid_t async_obsolete_data_read(int, void *, size_t, ipc_call_t *);
-extern int async_obsolete_data_read_start_generic(int, void *, size_t, int);
-
-extern int async_obsolete_data_write_start_generic(int, const void *, size_t, int);
-extern void async_obsolete_data_write_void(const int);
-
-extern int async_obsolete_forward_fast(ipc_callid_t, int, sysarg_t, sysarg_t,
-    sysarg_t, unsigned int);
-
-extern int async_obsolete_data_write_forward_fast(int, sysarg_t, sysarg_t, sysarg_t,
-    sysarg_t, sysarg_t, ipc_call_t *);
-
-#endif
-
-/** @}
- */
Index: uspace/lib/c/include/ns_obsolete.h
===================================================================
--- uspace/lib/c/include/ns_obsolete.h	(revision c69646f882a7a12ae81b85a589c3f95239f23e8d)
+++ 	(revision )
@@ -1,48 +1,0 @@
-/*
- * Copyright (c) 2006 Ondrej Palkovsky
- * All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * - Redistributions of source code must retain the above copyright
- *   notice, this list of conditions and the following disclaimer.
- * - Redistributions in binary form must reproduce the above copyright
- *   notice, this list of conditions and the following disclaimer in the
- *   documentation and/or other materials provided with the distribution.
- * - The name of the author may not be used to endorse or promote products
- *   derived from this software without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
- * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
- * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
- * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
- * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
- * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- */
-
-/** @addtogroup libc
- * @{
- */
-/** @file
- */
-
-#ifndef LIBC_NS_OBSOLETE_H_
-#define LIBC_NS_OBSOLETE_H_
-
-#include <sys/types.h>
-#include <task.h>
-
-extern int service_obsolete_connect(sysarg_t, sysarg_t, sysarg_t);
-extern int service_obsolete_connect_blocking(sysarg_t, sysarg_t,
-    sysarg_t);
-
-#endif
-
-/** @}
- */
Index: uspace/srv/hid/input/generic/input.c
===================================================================
--- uspace/srv/hid/input/generic/input.c	(revision c69646f882a7a12ae81b85a589c3f95239f23e8d)
+++ uspace/srv/hid/input/generic/input.c	(revision 5da719915787705f661397f14e439ebeb2aaf2f1)
@@ -47,7 +47,5 @@
 #include <stdio.h>
 #include <ns.h>
-#include <ns_obsolete.h>
 #include <async.h>
-#include <async_obsolete.h>
 #include <errno.h>
 #include <adt/fifo.h>
@@ -63,7 +61,4 @@
 #include <mouse.h>
 
-// FIXME: remove this header
-#include <abi/ipc/methods.h>
-
 #define NUM_LAYOUTS  3
 
@@ -77,5 +72,5 @@
 static void kbd_devs_reclaim(void);
 
-int client_phone = -1;
+async_sess_t *client_sess = NULL;
 
 /** List of keyboard devices */
@@ -86,5 +81,5 @@
 
 bool irc_service = false;
-int irc_phone = -1;
+async_sess_t *irc_sess = NULL;
 
 void kbd_push_data(kbd_dev_t *kdev, sysarg_t data)
@@ -170,6 +165,8 @@
 	
 	ev.c = layout_parse_ev(kdev->active_layout, &ev);
-	async_obsolete_msg_4(client_phone, INPUT_EVENT_KEY, ev.type, ev.key,
-	    ev.mods, ev.c);
+	
+	async_exch_t *exch = async_exchange_begin(client_sess);
+	async_msg_4(exch, INPUT_EVENT_KEY, ev.type, ev.key, ev.mods, ev.c);
+	async_exchange_end(exch);
 }
 
@@ -177,5 +174,7 @@
 void mouse_push_event_move(mouse_dev_t *mdev, int dx, int dy)
 {
-	async_obsolete_msg_2(client_phone, INPUT_EVENT_MOVE, dx, dy);
+	async_exch_t *exch = async_exchange_begin(client_sess);
+	async_msg_2(exch, INPUT_EVENT_MOVE, dx, dy);
+	async_exchange_end(exch);
 }
 
@@ -183,22 +182,21 @@
 void mouse_push_event_button(mouse_dev_t *mdev, int bnum, int press)
 {
-	async_obsolete_msg_2(client_phone, INPUT_EVENT_BUTTON, bnum, press);
+	async_exch_t *exch = async_exchange_begin(client_sess);
+	async_msg_2(exch, INPUT_EVENT_BUTTON, bnum, press);
+	async_exchange_end(exch);
 }
 
 static void client_connection(ipc_callid_t iid, ipc_call_t *icall, void *arg)
 {
-	ipc_callid_t callid;
-	ipc_call_t call;
-	int retval;
-	
 	async_answer_0(iid, EOK);
 	
 	while (true) {
-		callid = async_get_call(&call);
+		ipc_call_t call;
+		ipc_callid_t callid = async_get_call(&call);
 		
 		if (!IPC_GET_IMETHOD(call)) {
-			if (client_phone != -1) {
-				async_obsolete_hangup(client_phone);
-				client_phone = -1;
+			if (client_sess != NULL) {
+				async_hangup(client_sess);
+				client_sess = NULL;
 			}
 			
@@ -207,26 +205,26 @@
 		}
 		
-		switch (IPC_GET_IMETHOD(call)) {
-		case IPC_M_CONNECT_TO_ME:
-			if (client_phone != -1) {
-				retval = ELIMIT;
+		async_sess_t *sess =
+		    async_callback_receive_start(EXCHANGE_SERIALIZE, &call);
+		if (sess != NULL) {
+			if (client_sess == NULL) {
+				client_sess = sess;
+				async_answer_0(callid, EOK);
+			} else
+				async_answer_0(callid, ELIMIT);
+		} else {
+			switch (IPC_GET_IMETHOD(call)) {
+			case INPUT_YIELD:
+				kbd_devs_yield();
+				async_answer_0(callid, EOK);
 				break;
+			case INPUT_RECLAIM:
+				kbd_devs_reclaim();
+				async_answer_0(callid, EOK);
+				break;
+			default:
+				async_answer_0(callid, EINVAL);
 			}
-			client_phone = IPC_GET_ARG5(call);
-			retval = 0;
-			break;
-		case INPUT_YIELD:
-			kbd_devs_yield();
-			retval = 0;
-			break;
-		case INPUT_RECLAIM:
-			kbd_devs_reclaim();
-			retval = 0;
-			break;
-		default:
-			retval = EINVAL;
 		}
-		
-		async_answer_0(callid, retval);
 	}
 }
@@ -648,6 +646,7 @@
 	
 	if (irc_service) {
-		while (irc_phone < 0)
-			irc_phone = service_obsolete_connect_blocking(SERVICE_IRC, 0, 0);
+		while (irc_sess == NULL)
+			irc_sess = service_connect_blocking(EXCHANGE_SERIALIZE,
+			    SERVICE_IRC, 0, 0);
 	}
 	
Index: uspace/srv/hid/input/include/input.h
===================================================================
--- uspace/srv/hid/input/include/input.h	(revision c69646f882a7a12ae81b85a589c3f95239f23e8d)
+++ uspace/srv/hid/input/include/input.h	(revision 5da719915787705f661397f14e439ebeb2aaf2f1)
@@ -40,4 +40,5 @@
 
 #include <bool.h>
+#include <async.h>
 
 #define NAME       "input"
@@ -45,5 +46,5 @@
 
 extern bool irc_service;
-extern int irc_phone;
+extern async_sess_t *irc_sess;
 
 #endif
Index: uspace/srv/hid/input/port/ns16550.c
===================================================================
--- uspace/srv/hid/input/port/ns16550.c	(revision c69646f882a7a12ae81b85a589c3f95239f23e8d)
+++ uspace/srv/hid/input/port/ns16550.c	(revision 5da719915787705f661397f14e439ebeb2aaf2f1)
@@ -38,5 +38,4 @@
 #include <ipc/irc.h>
 #include <async.h>
-#include <async_obsolete.h>
 #include <sysinfo.h>
 #include <input.h>
@@ -158,7 +157,9 @@
 	kbd_push_data(kbd_dev, IPC_GET_ARG2(*call));
 	
-	if (irc_service)
-		async_obsolete_msg_1(irc_phone, IRC_CLEAR_INTERRUPT,
-		    IPC_GET_IMETHOD(*call));
+	if (irc_service) {
+		async_exch_t *exch = async_exchange_begin(irc_sess);
+		async_msg_1(exch, IRC_CLEAR_INTERRUPT, IPC_GET_IMETHOD(*call));
+		async_exchange_end(exch);
+	}
 }
 
Index: uspace/srv/hid/s3c24xx_ts/s3c24xx_ts.c
===================================================================
--- uspace/srv/hid/s3c24xx_ts/s3c24xx_ts.c	(revision c69646f882a7a12ae81b85a589c3f95239f23e8d)
+++ uspace/srv/hid/s3c24xx_ts/s3c24xx_ts.c	(revision 5da719915787705f661397f14e439ebeb2aaf2f1)
@@ -44,5 +44,4 @@
 #include <ipc/mouseev.h>
 #include <async.h>
-#include <async_obsolete.h>
 #include <unistd.h>
 #include <stdio.h>
@@ -53,9 +52,6 @@
 #include "s3c24xx_ts.h"
 
-// FIXME: remove this header
-#include <abi/ipc/methods.h>
-
-#define NAME "s3c24ser"
-#define NAMESPACE "hid"
+#define NAME       "s3c24ser"
+#define NAMESPACE  "hid"
 
 static irq_cmd_t ts_irq_cmds[] = {
@@ -134,5 +130,5 @@
 
 	ts->io = vaddr;
-	ts->client_phone = -1;
+	ts->client_sess = NULL;
 	ts->state = ts_wait_pendown;
 	ts->last_x = 0;
@@ -284,5 +280,8 @@
 	button = 1;
 	press = 0;
-	async_obsolete_msg_2(ts->client_phone, MOUSEEV_BUTTON_EVENT, button, press);
+	
+	async_exch_t *exch = async_exchange_begin(ts->client_sess);
+	async_msg_2(exch, MOUSEEV_BUTTON_EVENT, button, press);
+	async_exchange_end(exch);
 
 	s3c24xx_ts_wait_for_int_mode(ts, updn_down);
@@ -325,6 +324,8 @@
 
 	/* Send notifications to client. */
-	async_obsolete_msg_2(ts->client_phone, MOUSEEV_MOVE_EVENT, dx, dy);
-	async_obsolete_msg_2(ts->client_phone, MOUSEEV_BUTTON_EVENT, button, press);
+	async_exch_t *exch = async_exchange_begin(ts->client_sess);
+	async_msg_2(exch, MOUSEEV_MOVE_EVENT, dx, dy);
+	async_msg_2(exch, MOUSEEV_BUTTON_EVENT, button, press);
+	async_exchange_end(exch);
 
 	ts->last_x = x_pos;
@@ -377,36 +378,30 @@
     void *arg)
 {
-	ipc_callid_t callid;
-	ipc_call_t call;
-	int retval;
-
 	async_answer_0(iid, EOK);
-
-	while (1) {
-		callid = async_get_call(&call);
+	
+	while (true) {
+		ipc_call_t call;
+		ipc_callid_t callid = async_get_call(&call);
 		
 		if (!IPC_GET_IMETHOD(call)) {
-			if (ts->client_phone != -1) {
-				async_obsolete_hangup(ts->client_phone);
-				ts->client_phone = -1;
+			if (ts->client_sess != NULL) {
+				async_hangup(ts->client_sess);
+				ts->client_sess = NULL;
 			}
-
+			
 			async_answer_0(callid, EOK);
 			return;
 		}
 		
-		switch (IPC_GET_IMETHOD(call)) {
-		case IPC_M_CONNECT_TO_ME:
-			if (ts->client_phone != -1) {
-				retval = ELIMIT;
-				break;
-			}
-			ts->client_phone = IPC_GET_ARG5(call);
-			retval = 0;
-			break;
-		default:
-			retval = EINVAL;
-		}
-		async_answer_0(callid, retval);
+		async_sess_t *sess =
+		    async_callback_receive_start(EXCHANGE_SERIALIZE, &call);
+		if (sess != NULL) {
+			if (ts->client_sess == NULL) {
+				ts->client_sess = sess;
+				async_answer_0(callid, EOK);
+			} else
+				async_answer_0(callid, ELIMIT);
+		} else
+			async_answer_0(callid, EINVAL);
 	}
 }
Index: uspace/srv/hid/s3c24xx_ts/s3c24xx_ts.h
===================================================================
--- uspace/srv/hid/s3c24xx_ts/s3c24xx_ts.h	(revision c69646f882a7a12ae81b85a589c3f95239f23e8d)
+++ uspace/srv/hid/s3c24xx_ts/s3c24xx_ts.h	(revision 5da719915787705f661397f14e439ebeb2aaf2f1)
@@ -39,4 +39,5 @@
 
 #include <sys/types.h>
+#include <async.h>
 
 /** S3C24xx ADC and touch-screen I/O */
@@ -121,6 +122,6 @@
 	s3c24xx_adc_io_t *io;
 
-	/** Callback phone to the client */
-	int client_phone;
+	/** Callback session to the client */
+	async_sess_t *client_sess;
 
 	/** Service ID */
Index: uspace/srv/hw/bus/cuda_adb/cuda_adb.c
===================================================================
--- uspace/srv/hw/bus/cuda_adb/cuda_adb.c	(revision c69646f882a7a12ae81b85a589c3f95239f23e8d)
+++ uspace/srv/hw/bus/cuda_adb/cuda_adb.c	(revision 5da719915787705f661397f14e439ebeb2aaf2f1)
@@ -48,12 +48,8 @@
 #include <ipc/adb.h>
 #include <async.h>
-#include <async_obsolete.h>
 #include <assert.h>
 #include "cuda_adb.h"
 
-// FIXME: remove this header
-#include <abi/ipc/methods.h>
-
-#define NAME "cuda_adb"
+#define NAME  "cuda_adb"
 
 static void cuda_connection(ipc_callid_t iid, ipc_call_t *icall, void *arg);
@@ -154,5 +150,5 @@
 
 	for (i = 0; i < ADB_MAX_ADDR; ++i) {
-		adb_dev[i].client_phone = -1;
+		adb_dev[i].client_sess = NULL;
 		adb_dev[i].service_id = 0;
 	}
@@ -199,5 +195,4 @@
 	sysarg_t method;
 	service_id_t dsid;
-	int retval;
 	int dev_addr, i;
 
@@ -220,5 +215,5 @@
 	async_answer_0(iid, EOK);
 
-	while (1) {
+	while (true) {
 		callid = async_get_call(&call);
 		method = IPC_GET_IMETHOD(call);
@@ -230,27 +225,24 @@
 		}
 		
-		switch (method) {
-		case IPC_M_CONNECT_TO_ME:
-			if (adb_dev[dev_addr].client_phone != -1) {
-				retval = ELIMIT;
-				break;
-			}
-			adb_dev[dev_addr].client_phone = IPC_GET_ARG5(call);
-			/*
-			 * A hack so that we send the data to the phone
-			 * regardless of which address the device is on.
-			 */
-			for (i = 0; i < ADB_MAX_ADDR; ++i) {
-				if (adb_dev[i].service_id == dsid) {
-					adb_dev[i].client_phone = IPC_GET_ARG5(call);
+		async_sess_t *sess =
+		    async_callback_receive_start(EXCHANGE_SERIALIZE, &call);
+		if (sess != NULL) {
+			if (adb_dev[dev_addr].client_sess == NULL) {
+				adb_dev[dev_addr].client_sess = sess;
+				
+				/*
+				 * A hack so that we send the data to the session
+				 * regardless of which address the device is on.
+				 */
+				for (i = 0; i < ADB_MAX_ADDR; ++i) {
+					if (adb_dev[i].service_id == dsid)
+						adb_dev[i].client_sess = sess;
 				}
-			}
-			retval = 0;
-			break;
-		default:
-			retval = EINVAL;
-			break;
-		}
-		async_answer_0(callid, retval);
+				
+				async_answer_0(callid, EOK);
+			} else
+				async_answer_0(callid, ELIMIT);
+		} else
+			async_answer_0(callid, EINVAL);
 	}
 }
@@ -483,8 +475,11 @@
 	reg_val = ((uint16_t) data[1] << 8) | (uint16_t) data[2];
 
-	if (adb_dev[dev_addr].client_phone == -1)
-		return;
-
-	async_obsolete_msg_1(adb_dev[dev_addr].client_phone, ADB_REG_NOTIF, reg_val);
+	if (adb_dev[dev_addr].client_sess == NULL)
+		return;
+
+	async_exch_t *exch =
+	    async_exchange_begin(adb_dev[dev_addr].client_sess);
+	async_msg_1(exch, ADB_REG_NOTIF, reg_val);
+	async_exchange_end(exch);
 }
 
Index: uspace/srv/hw/bus/cuda_adb/cuda_adb.h
===================================================================
--- uspace/srv/hw/bus/cuda_adb/cuda_adb.h	(revision c69646f882a7a12ae81b85a589c3f95239f23e8d)
+++ uspace/srv/hw/bus/cuda_adb/cuda_adb.h	(revision 5da719915787705f661397f14e439ebeb2aaf2f1)
@@ -38,5 +38,5 @@
 
 #include <sys/types.h>
-#include <ipc/loc.h>
+#include <async.h>
 #include <fibril_synch.h>
 
@@ -105,5 +105,5 @@
 typedef struct {
 	service_id_t service_id;
-	int client_phone;
+	async_sess_t *client_sess;
 } adb_dev_t;
 
Index: uspace/srv/hw/char/i8042/i8042.c
===================================================================
--- uspace/srv/hw/char/i8042/i8042.c	(revision c69646f882a7a12ae81b85a589c3f95239f23e8d)
+++ uspace/srv/hw/char/i8042/i8042.c	(revision 5da719915787705f661397f14e439ebeb2aaf2f1)
@@ -32,5 +32,5 @@
  * @ingroup kbd
  * @{
- */ 
+ */
 /** @file
  * @brief i8042 PS/2 port driver.
@@ -41,5 +41,4 @@
 #include <loc.h>
 #include <async.h>
-#include <async_obsolete.h>
 #include <unistd.h>
 #include <sysinfo.h>
@@ -49,9 +48,6 @@
 #include "i8042.h"
 
-// FIXME: remove this header
-#include <abi/ipc/methods.h>
-
-#define NAME "i8042"
-#define NAMESPACE "char"
+#define NAME       "i8042"
+#define NAMESPACE  "char"
 
 /* Interesting bits for status register */
@@ -145,5 +141,5 @@
 
 	for (i = 0; i < MAX_DEVS; i++) {
-		i8042_port[i].client_phone = -1;
+		i8042_port[i].client_sess = NULL;
 
 		snprintf(name, 16, "%s/ps2%c", NAMESPACE, dchar[i]);
@@ -257,24 +253,26 @@
 		}
 		
-		switch (method) {
-		case IPC_M_CONNECT_TO_ME:
-			printf(NAME ": creating callback connection\n");
-			if (i8042_port[dev_id].client_phone != -1) {
+		async_sess_t *sess =
+		    async_callback_receive_start(EXCHANGE_SERIALIZE, &call);
+		if (sess != NULL) {
+			if (i8042_port[dev_id].client_sess == NULL) {
+				i8042_port[dev_id].client_sess = sess;
+				retval = EOK;
+			} else
 				retval = ELIMIT;
+		} else {
+			switch (method) {
+			case IPC_FIRST_USER_METHOD:
+				printf(NAME ": write %" PRIun " to devid %d\n",
+				    IPC_GET_ARG1(call), dev_id);
+				i8042_port_write(dev_id, IPC_GET_ARG1(call));
+				retval = 0;
+				break;
+			default:
+				retval = EINVAL;
 				break;
 			}
-			i8042_port[dev_id].client_phone = IPC_GET_ARG5(call);
-			retval = 0;
-			break;
-		case IPC_FIRST_USER_METHOD:
-			printf(NAME ": write %" PRIun " to devid %d\n",
-			    IPC_GET_ARG1(call), dev_id);
-			i8042_port_write(dev_id, IPC_GET_ARG1(call));
-			retval = 0;
-			break;
-		default:
-			retval = EINVAL;
-			break;
 		}
+		
 		async_answer_0(callid, retval);
 	}
@@ -305,7 +303,9 @@
 	}
 
-	if (i8042_port[devid].client_phone != -1) {
-		async_obsolete_msg_1(i8042_port[devid].client_phone,
-		    IPC_FIRST_USER_METHOD, data);
+	if (i8042_port[devid].client_sess != NULL) {
+		async_exch_t *exch =
+		    async_exchange_begin(i8042_port[devid].client_sess);
+		async_msg_1(exch, IPC_FIRST_USER_METHOD, data);
+		async_exchange_end(exch);
 	}
 }
Index: uspace/srv/hw/char/i8042/i8042.h
===================================================================
--- uspace/srv/hw/char/i8042/i8042.h	(revision c69646f882a7a12ae81b85a589c3f95239f23e8d)
+++ uspace/srv/hw/char/i8042/i8042.h	(revision 5da719915787705f661397f14e439ebeb2aaf2f1)
@@ -39,6 +39,7 @@
 #define i8042_H_
 
+#include <sys/types.h>
 #include <libarch/ddi.h>
-#include <libarch/types.h>
+#include <async.h>
 
 /** i8042 HW I/O interface */
@@ -53,5 +54,5 @@
 typedef struct {
 	service_id_t service_id;
-	int client_phone;
+	async_sess_t *client_sess;
 } i8042_port_t;
 
@@ -60,3 +61,3 @@
 /**
  * @}
- */ 
+ */
Index: uspace/srv/hw/char/s3c24xx_uart/s3c24xx_uart.c
===================================================================
--- uspace/srv/hw/char/s3c24xx_uart/s3c24xx_uart.c	(revision c69646f882a7a12ae81b85a589c3f95239f23e8d)
+++ uspace/srv/hw/char/s3c24xx_uart/s3c24xx_uart.c	(revision 5da719915787705f661397f14e439ebeb2aaf2f1)
@@ -42,5 +42,4 @@
 #include <ipc/char.h>
 #include <async.h>
-#include <async_obsolete.h>
 #include <unistd.h>
 #include <stdio.h>
@@ -51,9 +50,6 @@
 #include "s3c24xx_uart.h"
 
-// FIXME: remove this header
-#include <abi/ipc/methods.h>
-
-#define NAME "s3c24ser"
-#define NAMESPACE "char"
+#define NAME       "s3c24ser"
+#define NAMESPACE  "char"
 
 static irq_cmd_t uart_irq_cmds[] = {
@@ -117,15 +113,11 @@
     void *arg)
 {
-	ipc_callid_t callid;
-	ipc_call_t call;
-	sysarg_t method;
-	int retval;
-
 	/* Answer the IPC_M_CONNECT_ME_TO call. */
 	async_answer_0(iid, EOK);
-
-	while (1) {
-		callid = async_get_call(&call);
-		method = IPC_GET_IMETHOD(call);
+	
+	while (true) {
+		ipc_call_t call;
+		ipc_callid_t callid = async_get_call(&call);
+		sysarg_t method = IPC_GET_IMETHOD(call);
 		
 		if (!method) {
@@ -135,21 +127,24 @@
 		}
 		
-		switch (method) {
-		case IPC_M_CONNECT_TO_ME:
-			printf(NAME ": creating callback connection\n");
-			uart->client_phone = IPC_GET_ARG5(call);
-			retval = 0;
-			break;
-		case CHAR_WRITE_BYTE:
-			printf(NAME ": write %" PRIun " to device\n",
-			    IPC_GET_ARG1(call));
-			s3c24xx_uart_sendb(uart, (uint8_t) IPC_GET_ARG1(call));
-			retval = 0;
-			break;
-		default:
-			retval = EINVAL;
-			break;
+		async_sess_t *sess =
+		    async_callback_receive_start(EXCHANGE_SERIALIZE, &call);
+		if (sess != NULL) {
+			if (uart->client_sess == NULL) {
+				uart->client_sess = sess;
+				async_answer_0(callid, EOK);
+			} else
+				async_answer_0(callid, ELIMIT);
+		} else {
+			switch (method) {
+			case CHAR_WRITE_BYTE:
+				printf(NAME ": write %" PRIun " to device\n",
+				    IPC_GET_ARG1(call));
+				s3c24xx_uart_sendb(uart, (uint8_t) IPC_GET_ARG1(call));
+				async_answer_0(callid, EOK);
+				break;
+			default:
+				async_answer_0(callid, EINVAL);
+			}
 		}
-		async_answer_0(callid, retval);
 	}
 }
@@ -163,7 +158,8 @@
 		uint32_t status = pio_read_32(&uart->io->uerstat);
 
-		if (uart->client_phone != -1) {
-			async_obsolete_msg_1(uart->client_phone, CHAR_NOTIF_BYTE,
-			    data);
+		if (uart->client_sess != NULL) {
+			async_exch_t *exch = async_exchange_begin(uart->client_sess);
+			async_msg_1(exch, CHAR_NOTIF_BYTE, data);
+			async_exchange_end(exch);
 		}
 
@@ -191,5 +187,5 @@
 
 	uart->io = vaddr;
-	uart->client_phone = -1;
+	uart->client_sess = NULL;
 
 	printf(NAME ": device at physical address %p, inr %" PRIun ".\n",
Index: uspace/srv/hw/char/s3c24xx_uart/s3c24xx_uart.h
===================================================================
--- uspace/srv/hw/char/s3c24xx_uart/s3c24xx_uart.h	(revision c69646f882a7a12ae81b85a589c3f95239f23e8d)
+++ uspace/srv/hw/char/s3c24xx_uart/s3c24xx_uart.h	(revision 5da719915787705f661397f14e439ebeb2aaf2f1)
@@ -39,4 +39,5 @@
 
 #include <sys/types.h>
+#include <async.h>
 
 /** S3C24xx UART I/O */
@@ -84,6 +85,6 @@
 	s3c24xx_uart_io_t *io;
 
-	/** Callback phone to the client */
-	int client_phone;
+	/** Callback session to the client */
+	async_sess_t *client_sess;
 
 	/** Service ID */

Index: uspace/drv/bus/usb/usbhid/kbd/kbddev.c
===================================================================
--- uspace/drv/bus/usb/usbhid/kbd/kbddev.c	(revision c69646f882a7a12ae81b85a589c3f95239f23e8d)
+++ uspace/drv/bus/usb/usbhid/kbd/kbddev.c	(revision 5da719915787705f661397f14e439ebeb2aaf2f1)
@@ -43,5 +43,4 @@
 #include <ipc/kbdev.h>
 #include <async.h>
-#include <async_obsolete.h>
 #include <fibril.h>
 #include <fibril_synch.h>
@@ -70,7 +69,4 @@
 
 #include "../usbhid.h"
-
-// FIXME: remove this header
-#include <abi/ipc/methods.h>
 
 /*----------------------------------------------------------------------------*/
@@ -167,5 +163,5 @@
  *
  * Currently recognizes only one method (IPC_M_CONNECT_TO_ME), in which case it
  * assumes the caller is the console and thus it stores IPC phone to it for 
  * later use by the driver to notify about key events.
  *
@@ -178,7 +174,6 @@
 {
 	sysarg_t method = IPC_GET_IMETHOD(*icall);
-	int callback;
-	
-	usb_kbd_t *kbd_dev = (usb_kbd_t *)fun->driver_data;
+	
+	usb_kbd_t *kbd_dev = (usb_kbd_t *) fun->driver_data;
 	if (kbd_dev == NULL) {
 		usb_log_debug("default_connection_handler: "
@@ -187,30 +182,29 @@
 		return;
 	}
-
-	switch (method) {
-	case IPC_M_CONNECT_TO_ME:
-		callback = IPC_GET_ARG5(*icall);
-
-		if (kbd_dev->console_phone != -1) {
+	
+	async_sess_t *sess =
+	    async_callback_receive_start(EXCHANGE_SERIALIZE, icall);
+	if (sess != NULL) {
+		if (kbd_dev->console_sess == NULL) {
+			kbd_dev->console_sess = sess;
+			usb_log_debug("default_connection_handler: OK\n");
+			async_answer_0(icallid, EOK);
+		} else {
 			usb_log_debug("default_connection_handler: "
-			    "console phone already set\n");
+			    "console session already set\n");
 			async_answer_0(icallid, ELIMIT);
-			return;
-		}
-
-		kbd_dev->console_phone = callback;
-		
-		usb_log_debug("default_connection_handler: OK\n");
-		async_answer_0(icallid, EOK);
-		break;
-	case KBDEV_SET_IND:
-		kbd_dev->mods = IPC_GET_ARG1(*icall);
-		usb_kbd_set_led(kbd_dev->hid_dev, kbd_dev);
-		async_answer_0(icallid, EOK);
-		break;
-	default:
-		usb_log_debug("default_connection_handler: Wrong function.\n");
-		async_answer_0(icallid, EINVAL);
-		break;
+		}
+	} else {
+		switch (method) {
+		case KBDEV_SET_IND:
+			kbd_dev->mods = IPC_GET_ARG1(*icall);
+			usb_kbd_set_led(kbd_dev->hid_dev, kbd_dev);
+			async_answer_0(icallid, EOK);
+			break;
+		default:
+			usb_log_debug("default_connection_handler: Wrong function.\n");
+			async_answer_0(icallid, EINVAL);
+			break;
+		}
 	}
 }
@@ -301,5 +295,5 @@
 {
 	usb_log_debug2("Sending kbdev event %d/%d to the console\n", type, key);
-	if (kbd_dev->console_phone < 0) {
+	if (kbd_dev->console_sess == NULL) {
 		usb_log_warning(
 		    "Connection to console not ready, key discarded.\n");
@@ -307,5 +301,7 @@
 	}
 	
-	async_obsolete_msg_2(kbd_dev->console_phone, KBDEV_EVENT, type, key);
+	async_exch_t *exch = async_exchange_begin(kbd_dev->console_sess);
+	async_msg_2(exch, KBDEV_EVENT, type, key);
+	async_exchange_end(exch);
 }
 
@@ -510,5 +506,5 @@
 	}
 	
-	kbd_dev->console_phone = -1;
+	kbd_dev->console_sess = NULL;
 	kbd_dev->initialized = USB_KBD_STATUS_UNINITIALIZED;
 	
@@ -785,6 +781,6 @@
 	}
 	
-	// hangup phone to the console
-	async_obsolete_hangup(kbd_dev->console_phone);
+	// hangup session to the console
+	async_hangup(kbd_dev->console_sess);
 	
 	if (kbd_dev->repeat_mtx != NULL) {
@@ -167,5 +163,5 @@
  *
  * Currently recognizes only one method (IPC_M_CONNECT_TO_ME), in which case it
- * assumes the caller is the console and thus it stores IPC phone to it for 
+ * assumes the caller is the console and thus it stores IPC session to it for
  * later use by the driver to notify about key events.
  *
Index: uspace/drv/bus/usb/usbhid/mouse/mousedev.c
===================================================================
--- uspace/drv/bus/usb/usbhid/mouse/mousedev.c	(revision c69646f882a7a12ae81b85a589c3f95239f23e8d)
+++ uspace/drv/bus/usb/usbhid/mouse/mousedev.c	(revision 5da719915787705f661397f14e439ebeb2aaf2f1)
@@ -42,5 +42,4 @@
 #include <errno.h>
 #include <async.h>
-#include <async_obsolete.h>
 #include <str_error.h>
 #include <ipc/mouseev.h>
@@ -56,8 +55,5 @@
 #define ARROWS_PER_SINGLE_WHEEL 3
 
-// FIXME: remove this header
-#include <abi/ipc/methods.h>
-
-#define NAME "mouse"
+#define NAME  "mouse"
 
 /*----------------------------------------------------------------------------*/
@@ -128,7 +124,5 @@
     ipc_callid_t icallid, ipc_call_t *icall)
 {
-	sysarg_t method = IPC_GET_IMETHOD(*icall);
-	
-	usb_mouse_t *mouse_dev = (usb_mouse_t *)fun->driver_data;
+	usb_mouse_t *mouse_dev = (usb_mouse_t *) fun->driver_data;
 	
 	if (mouse_dev == NULL) {
@@ -141,28 +135,28 @@
 	usb_log_debug("default_connection_handler: fun->name: %s\n",
 	              fun->name);
-	usb_log_debug("default_connection_handler: mouse_phone: %d, wheel "
-	    "phone: %d\n", mouse_dev->mouse_phone, mouse_dev->wheel_phone);
-	
-	int *phone = (str_cmp(fun->name, HID_MOUSE_FUN_NAME) == 0) 
-		     ? &mouse_dev->mouse_phone : &mouse_dev->wheel_phone;
-	
-	if (method == IPC_M_CONNECT_TO_ME) {
-		int callback = IPC_GET_ARG5(*icall);
-
-		if (*phone != -1) {
+	usb_log_debug("default_connection_handler: mouse_sess: %p, "
+	    "wheel_sess: %p\n", mouse_dev->mouse_sess, mouse_dev->wheel_sess);
+	
+	async_sess_t **sess_ptr =
+	    (str_cmp(fun->name, HID_MOUSE_FUN_NAME) == 0) ?
+	    &mouse_dev->mouse_sess : &mouse_dev->wheel_sess;
+	
+	async_sess_t *sess =
+	    async_callback_receive_start(EXCHANGE_SERIALIZE, icall);
+	if (sess != NULL) {
+		if (*sess_ptr == NULL) {
+			*sess_ptr = sess;
+			usb_log_debug("Console session to mouse set ok (%p).\n",
+			    sess);
+			async_answer_0(icallid, EOK);
+		} else {
 			usb_log_debug("default_connection_handler: Console "
-			    "phone to mouse already set.\n");
+			    "session to mouse already set.\n");
 			async_answer_0(icallid, ELIMIT);
-			return;
 		}
-
-		*phone = callback;
-		usb_log_debug("Console phone to mouse set ok (%d).\n", *phone);
-		async_answer_0(icallid, EOK);
-		return;
-	}
-
-	usb_log_debug("default_connection_handler: Invalid function.\n");
-	async_answer_0(icallid, EINVAL);
+	} else {
+		usb_log_debug("default_connection_handler: Invalid function.\n");
+		async_answer_0(icallid, EINVAL);
+	}
 }
 
@@ -175,6 +169,6 @@
 		return NULL;
 	}
-	mouse->mouse_phone = -1;
-	mouse->wheel_phone = -1;
+	mouse->mouse_sess = NULL;
+	mouse->wheel_sess = NULL;
 	
 	return mouse;
@@ -187,12 +181,10 @@
 	assert(mouse_dev != NULL);
 	
-	// hangup phone to the console
-	if (mouse_dev->mouse_phone >= 0) {
-		async_obsolete_hangup(mouse_dev->mouse_phone);
-	}
-	
-	if (mouse_dev->wheel_phone >= 0) {
-		async_obsolete_hangup(mouse_dev->wheel_phone);
-	}
+	// hangup session to the console
+	if (mouse_dev->mouse_sess != NULL)
+		async_hangup(mouse_dev->mouse_sess);
+	
+	if (mouse_dev->wheel_sess != NULL)
+		async_hangup(mouse_dev->wheel_sess);
 }
 
@@ -203,5 +195,5 @@
 	unsigned int key = (wheel > 0) ? KC_UP : KC_DOWN;
 
-	if (mouse_dev->wheel_phone < 0) {
+	if (mouse_dev->wheel_sess == NULL) {
 		usb_log_warning(
 		    "Connection to console not ready, wheel roll discarded.\n");
@@ -215,8 +207,11 @@
 		/* Send arrow press and release. */
 		usb_log_debug2("Sending key %d to the console\n", key);
-		async_obsolete_msg_4(mouse_dev->wheel_phone, KBDEV_EVENT,
-		    KEY_PRESS, key, 0, 0);
-		async_obsolete_msg_4(mouse_dev->wheel_phone, KBDEV_EVENT,
-		    KEY_RELEASE, key, 0, 0);
+		
+		async_exch_t *exch = async_exchange_begin(mouse_dev->wheel_sess);
+		
+		async_msg_4(exch, KBDEV_EVENT, KEY_PRESS, key, 0, 0);
+		async_msg_4(exch, KBDEV_EVENT, KEY_RELEASE, key, 0, 0);
+		
+		async_exchange_end(exch);
 	}
 }
@@ -253,6 +248,6 @@
 	assert(mouse_dev != NULL);
 	
-	if (mouse_dev->mouse_phone < 0) {
-		usb_log_warning(NAME " No console phone.\n");
+	if (mouse_dev->mouse_sess == NULL) {
+		usb_log_warning(NAME " No console session.\n");
 		return true;
 	}
@@ -266,11 +261,11 @@
 
 	if ((shift_x != 0) || (shift_y != 0)) {
-		async_obsolete_req_2_0(mouse_dev->mouse_phone,
-		    MOUSEEV_MOVE_EVENT, shift_x, shift_y);
-	}
-
-	if (wheel != 0) {
+		async_exch_t *exch = async_exchange_begin(mouse_dev->mouse_sess);
+		async_req_2_0(exch, MOUSEEV_MOVE_EVENT, shift_x, shift_y);
+		async_exchange_end(exch);
+	}
+	
+	if (wheel != 0)
 		usb_mouse_send_wheel(mouse_dev, wheel);
-	}
 	
 	/*
@@ -292,12 +287,18 @@
 		if (mouse_dev->buttons[field->usage - field->usage_minimum] == 0
 		    && field->value != 0) {
-			async_obsolete_req_2_0(mouse_dev->mouse_phone,
-			    MOUSEEV_BUTTON_EVENT, field->usage, 1);
+			async_exch_t *exch =
+			    async_exchange_begin(mouse_dev->mouse_sess);
+			async_req_2_0(exch, MOUSEEV_BUTTON_EVENT, field->usage, 1);
+			async_exchange_end(exch);
+			
 			mouse_dev->buttons[field->usage - field->usage_minimum]
 			    = field->value;
 		} else if (mouse_dev->buttons[field->usage - field->usage_minimum] != 0
 		    && field->value == 0) {
-			async_obsolete_req_2_0(mouse_dev->mouse_phone,
-			   MOUSEEV_BUTTON_EVENT, field->usage, 0);
+			async_exch_t *exch =
+			    async_exchange_begin(mouse_dev->mouse_sess);
+			async_req_2_0(exch, MOUSEEV_BUTTON_EVENT, field->usage, 0);
+			async_exchange_end(exch);
+			
 			mouse_dev->buttons[field->usage - field->usage_minimum] =
 			   field->value;
Index: uspace/drv/bus/usb/usbhid/multimedia/multimedia.c
===================================================================
--- uspace/drv/bus/usb/usbhid/multimedia/multimedia.c	(revision c69646f882a7a12ae81b85a589c3f95239f23e8d)
+++ uspace/drv/bus/usb/usbhid/multimedia/multimedia.c	(revision 5da719915787705f661397f14e439ebeb2aaf2f1)
@@ -47,5 +47,4 @@
 #include <errno.h>
 #include <async.h>
-#include <async_obsolete.h>
 #include <str_error.h>
 
@@ -53,8 +52,5 @@
 #include <io/console.h>
 
-// FIXME: remove this header
-#include <abi/ipc/methods.h>
-
-#define NAME "multimedia-keys"
+#define NAME  "multimedia-keys"
 
 /*----------------------------------------------------------------------------*/
@@ -69,6 +65,6 @@
 	/** Count of stored keys (i.e. number of keys in the report). */
 	//size_t key_count;	
-	/** IPC phone to the console device (for sending key events). */
-	int console_phone;
+	/** IPC session to the console device (for sending key events). */
+	async_sess_t *console_sess;
 } usb_multimedia_t;
 
@@ -79,5 +75,5 @@
  *
  * Currently recognizes only one method (IPC_M_CONNECT_TO_ME), in which case it
  * assumes the caller is the console and thus it stores IPC phone to it for 
  * later use by the driver to notify about key events.
  *
@@ -91,6 +87,4 @@
 	usb_log_debug(NAME " default_connection_handler()\n");
 	
-	sysarg_t method = IPC_GET_IMETHOD(*icall);
-	
 	usb_multimedia_t *multim_dev = (usb_multimedia_t *)fun->driver_data;
 	
@@ -99,20 +93,17 @@
 		return;
 	}
-
-	if (method == IPC_M_CONNECT_TO_ME) {
-		int callback = IPC_GET_ARG5(*icall);
-
-		if (multim_dev->console_phone != -1) {
+	
+	async_sess_t *sess =
+	    async_callback_receive_start(EXCHANGE_SERIALIZE, icall);
+	if (sess != NULL) {
+		if (multim_dev->console_sess == NULL) {
+			multim_dev->console_sess = sess;
+			usb_log_debug(NAME " Saved session to console: %p\n",
+			    sess);
+			async_answer_0(icallid, EOK);
+		} else
 			async_answer_0(icallid, ELIMIT);
-			return;
-		}
-
-		multim_dev->console_phone = callback;
-		usb_log_debug(NAME " Saved phone to console: %d\n", callback);
-		async_answer_0(icallid, EOK);
-		return;
-	}
-	
-	async_answer_0(icallid, EINVAL);
+	} else
+		async_answer_0(icallid, EINVAL);
 }
 
@@ -155,5 +146,5 @@
 
 	usb_log_debug2(NAME " Sending key %d to the console\n", ev.key);
-	if (multim_dev->console_phone < 0) {
+	if (multim_dev->console_sess == NULL) {
 		usb_log_warning(
 		    "Connection to console not ready, key discarded.\n");
@@ -161,6 +152,7 @@
 	}
 	
-	async_obsolete_msg_4(multim_dev->console_phone, KBDEV_EVENT, ev.type, ev.key, 
-	    ev.mods, ev.c);
+	async_exch_t *exch = async_exchange_begin(multim_dev->console_sess);
+	async_msg_4(exch, KBDEV_EVENT, ev.type, ev.key, ev.mods, ev.c);
+	async_exchange_end(exch);
 }
 
@@ -222,5 +214,5 @@
 	}
 	
-	multim_dev->console_phone = -1;
+	multim_dev->console_sess = NULL;
 	
 	/*! @todo Autorepeat */
@@ -250,6 +242,6 @@
 	if (data != NULL) {
 		usb_multimedia_t *multim_dev = (usb_multimedia_t *)data;
-		// hangup phone to the console
-		async_obsolete_hangup(multim_dev->console_phone);
+		// hangup session to the console
+		async_hangup(multim_dev->console_sess);
 	}
 }
@@ -79,5 +75,5 @@
  *
  * Currently recognizes only one method (IPC_M_CONNECT_TO_ME), in which case it
- * assumes the caller is the console and thus it stores IPC phone to it for 
+ * assumes the caller is the console and thus it stores IPC session to it for
  * later use by the driver to notify about key events.
  *